@@ -17,13 +17,14 @@
 
 /** @file */
 
-#define DECLARE_BUILTIN_VARIABLE(x)	class x : public DynamicVariable { \
+#define DECLARE_BUILTIN_VARIABLE(x, i)	class x : public DynamicVariable { \
 					  private: \
-						void calculate() const;	\
+						void calculate(MathStructure &m) const;	\
  					  public: \
 						x(); \
 						x(const x *variable) {set(variable);} \
 						ExpressionItem *copy() const {return new x(this);} \
+						int id() const {return i;} \
 					};
 
 /// Type assumption.
@@ -39,7 +40,8 @@ typedef enum {
 	ASSUMPTION_TYPE_COMPLEX = 3,
 	ASSUMPTION_TYPE_REAL = 4,
 	ASSUMPTION_TYPE_RATIONAL = 5,
-	ASSUMPTION_TYPE_INTEGER = 6
+	ASSUMPTION_TYPE_INTEGER = 6,
+	ASSUMPTION_TYPE_BOOLEAN = 7
 } AssumptionType;
 
 /// Signedness assumption.
@@ -76,7 +78,7 @@ typedef enum {
 class Assumptions {
 
   protected:
-  
+
 	AssumptionType i_type;
 	AssumptionSign i_sign;
 	Number *fmin, *fmax;
@@ -92,6 +94,7 @@ class Assumptions {
 	bool isNonNegative();
 	bool isNonPositive();
 	bool isInteger();
+	bool isBoolean();
 	bool isNumber();
 	bool isRational();
 	bool isReal();
@@ -99,21 +102,21 @@ class Assumptions {
 	bool isNonZero();
 	bool isNonMatrix();
 	bool isScalar();
-	
+
 	AssumptionType type();
 	AssumptionSign sign();
 	void setType(AssumptionType ant);
 	void setSign(AssumptionSign as);
-	
-	void setMin(const Number *nmin);	
+
+	void setMin(const Number *nmin);
 	void setIncludeEqualsMin(bool include_equals);
-	bool includeEqualsMin() const;	
+	bool includeEqualsMin() const;
 	const Number *min() const;
-	void setMax(const Number *nmax);	
+	void setMax(const Number *nmax);
 	void setIncludeEqualsMax(bool include_equals);
-	bool includeEqualsMax() const;	
+	bool includeEqualsMax() const;
 	const Number *max() const;
-	
+
 };
 
 /// Abstract base class for variables.
@@ -123,7 +126,7 @@ class Variable : public ExpressionItem {
 
   public:
 
-	Variable(string cat_, string name_, string title_ = "", bool is_local = true, bool is_builtin = false, bool is_active = true);
+	Variable(std::string cat_, std::string name_, std::string title_ = "", bool is_local = true, bool is_builtin = false, bool is_active = true);
 	Variable();
 	Variable(const Variable *variable);
 	virtual ~Variable();
@@ -162,7 +165,9 @@ class Variable : public ExpressionItem {
 	virtual bool representsBoolean() {return false;}
 	virtual bool representsNonMatrix() {return false;}
 	virtual bool representsScalar() {return false;}
-	
+
+	virtual int id() const {return 0;}
+
 };
 
 /// A variable with unknown value.
@@ -171,10 +176,10 @@ class Variable : public ExpressionItem {
 class UnknownVariable : public Variable {
 
   protected:
-  
+
   	Assumptions *o_assumption;
   	MathStructure *mstruct;
-  
+
   public:
 
 	/** Create an unknown.
@@ -186,7 +191,7 @@ class UnknownVariable : public Variable {
 	* @param is_builtin If the variable is builtin and not modifiable.
 	* @param is_active If the variable is active and can be used in expressions.
 	*/
-	UnknownVariable(string cat_, string name_, string title_ = "", bool is_local = true, bool is_builtin = false, bool is_active = true);
+	UnknownVariable(std::string cat_, std::string name_, std::string title_ = "", bool is_local = true, bool is_builtin = false, bool is_active = true);
 	/** Create an empty unknown variable.
 	*/
 	UnknownVariable();
@@ -199,7 +204,7 @@ class UnknownVariable : public Variable {
 	virtual ExpressionItem *copy() const;
 	virtual void set(const ExpressionItem *item);
 	bool isKnown() const {return false;}
-	
+
 	/** Sets the assumptions of the unknown variable.
 	*
 	* @param ass Assumptions.
@@ -211,10 +216,10 @@ class UnknownVariable : public Variable {
 	* @returns Assumptions of the unknown variable.
 	*/
 	Assumptions *assumptions();
-	
+
 	const MathStructure &interval() const;
 	void setInterval(const MathStructure &o);
-	
+
 	virtual int subtype() const {return SUBTYPE_UNKNOWN_VARIABLE;}
 
 	virtual bool representsPositive(bool = false);
@@ -228,9 +233,10 @@ class UnknownVariable : public Variable {
 	virtual bool representsNonComplex(bool = false);
 	virtual bool representsComplex(bool = false);
 	virtual bool representsNonZero(bool = false);
+	virtual bool representsBoolean();
 	virtual bool representsNonMatrix();
 	virtual bool representsScalar();
-	
+
 };
 
 /// A variable with a known value.
@@ -243,15 +249,15 @@ class KnownVariable : public Variable {
 
   protected:
 
-	MathStructure *mstruct;
+	MathStructure *mstruct, *mstruct_alt;
 	bool b_expression;
  	int calculated_precision;
- 	bool calculated_with_interval, calculated_with_units;
-	string sexpression, suncertainty, sunit;
+	std::string sexpression, suncertainty, sunit;
+	bool b_relative_uncertainty;
 
   public:
-  
-	/** Create a known variable with a value. 
+
+	/** Create a known variable with a value.
 	*
 	* @param cat_ Category that the variable belongs to.
 	* @param name_ Initial name of the variable.
@@ -261,7 +267,7 @@ class KnownVariable : public Variable {
 	* @param is_builtin If the variable is builtin and not modifiable.
 	* @param is_active If the variable is active and can be used in expressions.
 	*/
-	KnownVariable(string cat_, string name_, const MathStructure &o, string title_ = "", bool is_local = true, bool is_builtin = false, bool is_active = true);
+	KnownVariable(std::string cat_, std::string name_, const MathStructure &o, std::string title_ = "", bool is_local = true, bool is_builtin = false, bool is_active = true);
 	/** Create a known variable with an text string expression.
 	*
 	* @param cat_ Category that the variable belongs to.
@@ -272,7 +278,7 @@ class KnownVariable : public Variable {
 	* @param is_builtin If the variable is builtin and not modifiable.
 	* @param is_active If the variable is active and can be used in expressions.
 	*/
-	KnownVariable(string cat_, string name_, string expression_, string title_ = "", bool is_local = true, bool is_builtin = false, bool is_active = true);	
+	KnownVariable(std::string cat_, std::string name_, std::string expression_, std::string title_ = "", bool is_local = true, bool is_builtin = false, bool is_active = true);
 	/** Create an empty known variable. Primarily for internal use.
 	*/
 	KnownVariable();
@@ -295,10 +301,10 @@ class KnownVariable : public Variable {
 	*
 	* @returns The variable's expression.
 	*/
-	virtual string expression() const;
-	virtual string uncertainty() const;
-	virtual string unit() const;
-	
+	virtual std::string expression() const;
+	virtual std::string uncertainty(bool *is_relative = NULL) const;
+	virtual std::string unit() const;
+
 	int subtype() const {return SUBTYPE_KNOWN_VARIABLE;}
 
 	/** Sets the value of the variable. If expression is set, it is cleared.
@@ -310,16 +316,16 @@ class KnownVariable : public Variable {
 	*
 	* @param expression_ Expression.
 	*/
-	virtual void set(string expression_);
-	virtual void setUncertainty(string standard_uncertainty);
-	virtual void setUnit(string unit_expression);
+	virtual void set(std::string expression_);
+	virtual void setUncertainty(std::string standard_uncertainty, bool is_relative = false);
+	virtual void setUnit(std::string unit_expression);
 
 	/** Returns the value of the variable. If no value is set or parsed and an expression is set, the expression is parsed and resulting value returned.
 	*
 	* @returns The value of the variable..
 	*/
 	virtual const MathStructure &get();
-	
+
 	virtual bool representsPositive(bool = false);
 	virtual bool representsNegative(bool = false);
 	virtual bool representsNonNegative(bool = false);
@@ -340,6 +346,8 @@ class KnownVariable : public Variable {
 	virtual bool representsNonMatrix();
 	virtual bool representsScalar();
 
+	virtual int id() const {return 0;}
+
 };
 
 /// Abstract base class for variables with a value which is recalculated when the precision has changed.
@@ -349,12 +357,12 @@ class DynamicVariable : public KnownVariable {
 
   protected:
 
-	virtual void calculate() const = 0;
+	virtual void calculate(MathStructure &m) const = 0;
 	bool always_recalculate;
-	
+
   public:
 
-	DynamicVariable(string cat_, string name_, string title_ = "", bool is_local = false, bool is_builtin = true, bool is_active = true);
+	DynamicVariable(std::string cat_, std::string name_, std::string title_ = "", bool is_local = false, bool is_builtin = true, bool is_active = true);
 	DynamicVariable(const DynamicVariable *variable);
 	DynamicVariable();
 	virtual ~DynamicVariable();
@@ -363,17 +371,16 @@ class DynamicVariable : public KnownVariable {
 	void set(const ExpressionItem *item);
 
 	const MathStructure &get();
-	
+
 	void set(const MathStructure &o);
-	void set(string expression_);
-	
+	void set(std::string expression_);
+
 	/** Returns the precision of the calculated value.
 	*
 	* @returns Precision of the calculated value or zero if the value has not yet been calculated.
 	*/
 	int calculatedPrecision() const;
-	bool calculatedWithInterval() const;
-	
+
 	virtual bool representsPositive(bool = false) {return true;}
 	virtual bool representsNegative(bool = false) {return false;}
 	virtual bool representsNonNegative(bool = false) {return true;}
@@ -394,29 +401,44 @@ class DynamicVariable : public KnownVariable {
 
 };
 
+enum {
+	VARIABLE_ID_E = 100,
+	VARIABLE_ID_PI = 101,
+	VARIABLE_ID_EULER = 102,
+	VARIABLE_ID_CATALAN = 103,
+	VARIABLE_ID_PRECISION = 140,
+	VARIABLE_ID_TODAY = 161,
+	VARIABLE_ID_TOMORROW = 162,
+	VARIABLE_ID_YESTERDAY = 163,
+	VARIABLE_ID_NOW = 164,
+	VARIABLE_ID_UPTIME = 201
+};
+
 /// Dynamic variable for Pi
-DECLARE_BUILTIN_VARIABLE(PiVariable)
+DECLARE_BUILTIN_VARIABLE(PiVariable, VARIABLE_ID_PI)
 /// Dynamic variable for e, the base of natural logarithms
-DECLARE_BUILTIN_VARIABLE(EVariable)
+DECLARE_BUILTIN_VARIABLE(EVariable, VARIABLE_ID_E)
 /// Dynamic variable for Euler's constant
-DECLARE_BUILTIN_VARIABLE(EulerVariable)
+DECLARE_BUILTIN_VARIABLE(EulerVariable, VARIABLE_ID_EULER)
 /// Dynamic variable for Catalan's constant
-DECLARE_BUILTIN_VARIABLE(CatalanVariable)
+DECLARE_BUILTIN_VARIABLE(CatalanVariable, VARIABLE_ID_CATALAN)
+
 /// Dynamic variable for current precision
 class PrecisionVariable : public DynamicVariable {
   private:
-	void calculate() const;
+	void calculate(MathStructure &m) const;
   public:
 	PrecisionVariable();
 	PrecisionVariable(const PrecisionVariable *variable) {set(variable);}
 	ExpressionItem *copy() const {return new PrecisionVariable(this);}
 	bool representsInteger(bool = false) {return true;}
 	bool representsNonInteger(bool = false) {return false;}
+	int id() const {return VARIABLE_ID_PRECISION;}
 };
 
 class TodayVariable : public DynamicVariable {
   private:
-	void calculate() const;
+	void calculate(MathStructure &m) const;
   public:
 	TodayVariable();
 	TodayVariable(const TodayVariable *variable) {set(variable);}
@@ -427,10 +449,11 @@ class TodayVariable : public DynamicVariable {
 	virtual bool representsNumber(bool b = false) {return b;}
 	virtual bool representsReal(bool b = false) {return b;}
 	virtual bool representsNonZero(bool b = false) {return b;}
+	int id() const {return VARIABLE_ID_TODAY;}
 };
 class TomorrowVariable : public DynamicVariable {
   private:
-	void calculate() const;
+	void calculate(MathStructure &m) const;
   public:
 	TomorrowVariable();
 	TomorrowVariable(const TomorrowVariable *variable) {set(variable);}
@@ -441,10 +464,11 @@ class TomorrowVariable : public DynamicVariable {
 	virtual bool representsNumber(bool b = false) {return b;}
 	virtual bool representsReal(bool b = false) {return b;}
 	virtual bool representsNonZero(bool b = false) {return b;}
+	int id() const {return VARIABLE_ID_TOMORROW;}
 };
 class YesterdayVariable : public DynamicVariable {
   private:
-	void calculate() const;
+	void calculate(MathStructure &m) const;
   public:
 	YesterdayVariable();
 	YesterdayVariable(const YesterdayVariable *variable) {set(variable);}
@@ -455,10 +479,11 @@ class YesterdayVariable : public DynamicVariable {
 	virtual bool representsNumber(bool b = false) {return b;}
 	virtual bool representsReal(bool b = false) {return b;}
 	virtual bool representsNonZero(bool b = false) {return b;}
+	int id() const {return VARIABLE_ID_YESTERDAY;}
 };
 class NowVariable : public DynamicVariable {
   private:
-	void calculate() const;
+	void calculate(MathStructure &m) const;
   public:
 	NowVariable();
 	NowVariable(const NowVariable *variable) {set(variable);}
@@ -469,6 +494,16 @@ class NowVariable : public DynamicVariable {
 	virtual bool representsNumber(bool b = false) {return b;}
 	virtual bool representsReal(bool b = false) {return b;}
 	virtual bool representsNonZero(bool b = false) {return b;}
+	int id() const {return VARIABLE_ID_NOW;}
+};
+class UptimeVariable : public DynamicVariable {
+  private:
+	void calculate(MathStructure &m) const;
+  public:
+	UptimeVariable();
+	UptimeVariable(const UptimeVariable *variable);
+	ExpressionItem *copy() const;
+	int id() const {return VARIABLE_ID_UPTIME;}
 };
 
 #endif
